import { _decorator,Node } from "cc";
import GBaseMode from "../GBaseMode";
import { GTactical } from "../entity/GTactical";
import { Prefab } from "cc";
import { instantiate } from "cc";
import { Vec2 } from "cc";
import { GRoleUtil } from "../entity/GRole";
import GRoleDefault from "../base/role/GRoleDefault";
import { v3 } from "cc";
import { TB } from "../../../resources/config/data/schema";
const { ccclass, property } = _decorator;

//PVP 角色
export enum GPVPModePlayerEnum{
    PLAYER, //玩家
    ENEMY, //敌人
}

//PVP 玩家信息
export interface GPVPModePlayerInfo{
    //阵法
    tactical: GTactical;
    //宠物列表
    roles: TB.TbGRole[];
}

/**
 * PVP 模式
 */
@ccclass('GPVPMode')
export default class GPVPMode extends GBaseMode<{}>{

    @property(Prefab)
    rolePrefab: Prefab = null;

    @property(Node)
    objects: Node = null;
    
    //玩家信息
    playerInfo: GPVPModePlayerInfo;
    //敌方信息
    enemyInfo: GPVPModePlayerInfo;

    //玩家宠物
    playerRoles: GRoleDefault[] = [];
    //敌方宠物
    enemyRoles: GRoleDefault[] = [];

    //玩家位置
    playerPos: Vec2 = new Vec2(-400,0);
    //敌方位置
    enemyPos: Vec2 = new Vec2(400,0);

    get scene():Node{
        return this.objects;
    }

    onLoad(){
        super.onLoad();
    }


    onSyncInitSuccess(): void {
                    
        //调整相机
        let camreaPos = this.camera.node.worldPosition;
        this.camera.node.worldPosition = v3(0,800,camreaPos.z)

        //初始化战斗
        console.log("GPVPMode 模式初始化");

        this.playerInfo = { tactical: GTactical.getTactical().setOffset(this.playerPos), roles: GRoleUtil.getGRoles([10004,10001,10004,10002,10003,10003]) };
        this.enemyInfo = { tactical: GTactical.getTactical(true).setOffset(this.enemyPos), roles: GRoleUtil.getGRoles([10002,10002,10001,10003,10004,10003]) };
        
        //生成玩家
        this.playerInfo.roles.forEach((info,index) => this.onGenRole(GPVPModePlayerEnum.PLAYER,index+1,info))
        this.enemyInfo.roles.forEach((info,index) => this.onGenRole(GPVPModePlayerEnum.ENEMY,index+1,info))

    }

    //生成角色
    onGenRole(type: GPVPModePlayerEnum,index:number,info:TB.TbGRole) {

        let tactical = this.getInfo(type).tactical;
        let pos:Vec2 = this.getInfo(type).tactical.getPosition(index);
        if(!pos) return;
        let role = instantiate(this.rolePrefab);
        let entity = role.getComponent(GRoleDefault);
        //初始化
        entity.onInit(type,info,tactical,index);

        //绑定寻敌
        entity.onQueryEunmy = () => {
            return this.getEnumy(entity,type);
        }

        this.addGObject(entity,tactical.getPosition(index));
        this.getOnesRole(type).push(entity);

    }

    //获取配置
    getInfo(type: GPVPModePlayerEnum): GPVPModePlayerInfo {
        if(type == GPVPModePlayerEnum.PLAYER) return this.playerInfo;
        if(type == GPVPModePlayerEnum.ENEMY) return this.enemyInfo;
    }

    //获取阵营角色
    getOnesRole(type: GPVPModePlayerEnum):GRoleDefault[]{
        if(type == GPVPModePlayerEnum.PLAYER) return this.playerRoles;
        if(type == GPVPModePlayerEnum.ENEMY) return this.enemyRoles;
    }

    //获取敌人
    getEnumy(player:GRoleDefault,type:GPVPModePlayerEnum):GRoleDefault{

        let enumyOnes = GPVPModePlayerEnum.ENEMY
        //如果是ENEMY 则 它的敌人是 PLAYER
        if(type == GPVPModePlayerEnum.ENEMY) enumyOnes = GPVPModePlayerEnum.PLAYER

        //获取敌人
        let roles = this.getOnesRole(enumyOnes);

        //返回敌人
        //获取我在第几排
        let playerXY = player.tactical.getXY(player.tacticalIndex);
        //通过排数获取最近的敌人
        let sort = roles.filter(role => !!role.get()).sort((enumy1,enumy2) => {
            let enumy1XY = enumy1.tactical.getXY(enumy1.tacticalIndex);
            let enumy2XY = enumy2.tactical.getXY(enumy2.tacticalIndex);
            return Math.abs((playerXY.y * 1000) - (enumy1XY.y * 1000)) + Math.abs((playerXY.x - enumy1XY.x)) -
                    Math.abs((playerXY.y * 1000) - (enumy2XY.y * 1000)) + Math.abs((playerXY.x - enumy2XY.x))
        });
        return sort[0]

    }
    
}


