package cn.jisol.game.actions.onhook;

import cn.jisol.game.actions.GActionEnum;
import cn.jisol.game.controller.game.GPlayerTacticalController;
import cn.jisol.game.entity.game.PlayerTactical;
import cn.jisol.game.network.client.GClient;
import cn.jisol.game.proto.GPVPMessage;
import cn.jisol.game.service.PlayerTacticalService;
import cn.jisol.ngame.actions.core.NAction;
import cn.jisol.ngame.actions.core.NActionMethod;
import cn.jisol.ngame.util.spring.SpringBeanUtils;

import java.util.*;

//PVP 模式
@NAction
public class GPVPAction {

    //当前等待PVP的玩家
    static LinkedList<GClient> pool = new LinkedList<>();

    //等待通知定时器
    static Timer timer;

    static {

        timer = new Timer();
        timer.schedule(new TimerTask() {
            @Override
            public void run() {
                GPVPAction.onWaitTimer();
            }
        },0,1000);

    }

    //加入PVP
    @NActionMethod(GActionEnum.S_MODE_PVP_JOIN)
    public static synchronized void onJoinPVP(GClient client){

        //如果加入过 PVP 则返回
        if(pool.contains(client)) return;

        for (int i = 0; i < pool.size(); i++) {
            GClient item = pool.get(i);
            if (item.player.getPlayerId().equals(client.player.getPlayerId())){
                //移除玩家
                pool.remove(item);
            }
        }

        //如果没有加入则加入
        pool.add(client);

        //通知开始等待
        client.invoke(GActionEnum.C_MODE_PVP_START_WAIT);

        GPVPAction.onUpdateMatchGame();

    }

    //离开PVP
    @NActionMethod(GActionEnum.S_MODE_PVP_LEAVE)
    public static void onJoinLeave(GClient client){

        //如果加入过 则 离开
        if(pool.contains(client)) {
            pool.remove(client);
            client.invoke(GActionEnum.C_MODE_PVP_END_WAIT);
        }

    }

    //刷新匹配
    public static void onUpdateMatchGame(){

        //判断是否有两人在等待 有 则两人匹配成功
        while (pool.size() >= 2){
            //匹配成功
            GClient client1 = pool.poll();
            GClient client2 = pool.poll();

            if(Objects.isNull(client1) || Objects.isNull(client2)) continue;
            //确保玩家都在线
            if(!(client1.isOpen() &&  client2.isOpen())){
                if(client2.isOpen()) pool.addFirst(client2);
                if(client1.isOpen()) pool.addFirst(client1);
                continue;
            }

            //获取双方的阵型
            GPlayerTacticalController tactical = SpringBeanUtils.getBean(GPlayerTacticalController.class);

            //构建匹配信息
            GPVPMessage.GPVPStart info = GPVPMessage.GPVPStart.newBuilder()
                    .setLeftTactical(tactical.getInfo(client1.player).data.getTacticalData())
                    .setRightTactical(tactical.getInfo(client2.player).data.getTacticalData())
                    .build();

            //取消等待
            client1.invoke(GActionEnum.C_MODE_PVP_END_WAIT);
            client2.invoke(GActionEnum.C_MODE_PVP_END_WAIT);

            //PVP 开始
            client1.invoke(GActionEnum.C_MODE_PVP_START, info);
            client2.invoke(GActionEnum.C_MODE_PVP_START, info);

        }

    }

    //通知玩家等待定时器
    public static void onWaitTimer(){
        //向所有玩家发送等待
        pool.forEach(client -> {
            client.invoke(GActionEnum.C_MODE_PVP_WAIT);
        });
    }

}
