package cn.jisol.game.network;

import cn.jisol.game.actions.GActionEnum;
import cn.jisol.game.controller.game.GPlayerController;
import cn.jisol.game.network.client.GClient;
import cn.jisol.ngame.network.JNetwork;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;

import javax.websocket.OnClose;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.ServerEndpoint;
import java.io.InputStream;
import java.util.Map;
import java.util.Objects;
import java.util.concurrent.ConcurrentHashMap;

@ServerEndpoint(
    value = "/websocket/{token}"
)
@Controller
public class WebSocket {

    static GPlayerController playerController;

    @Autowired
    private void setPlayerController(GPlayerController playerController){
        WebSocket.playerController = playerController;
    }

    public static final Map<String, GClient> CLIENTS = new ConcurrentHashMap<>();

    @OnOpen
    public void onOpen(Session session){

        String token = session.getPathParameters().get("token");
        GClient client = new GClient(token,session);

        if(Objects.isNull(client.user)){
            //发送Token过期请求
            client.invoke(GActionEnum.TOKEN_EXPIRED);
            //关闭连接
            client.Close();
            return;
        }

        client.player = playerController.getPlayerInfo(client.user).data;
        CLIENTS.put(session.getId(),client);
        System.out.printf("[WebSocket] %s 连接成功.\n",session.getId());

    }

    @OnMessage
    public void onMessage(Session session, InputStream inputStream){
        GClient client = CLIENTS.get(session.getId());
        JNetwork.onMessage(inputStream,client,client,CLIENTS,client.user,client.player);
    }

    @OnClose
    public void onClose(Session session){
        CLIENTS.remove(session.getId());
    }

}
